/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        ConfigurationManager: a class that loads configurations from a plain text file or a classad file

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: ConfigurationManager.h,v 1.12.4.1 2008/06/17 08:25:33 adorigo Exp $
*/
#ifndef _GLITE_CE_CREAM_CLIENT_UTIL_CONFMGR_H_
#define _GLITE_CE_CREAM_CLIENT_UTIL_CONFMGR_H_

#include <map>
#include <string>
#include <vector>
#include <cstdio>
#include "glite/ce/cream-client-api-c/file_ex.h"
#include "glite/ce/cream-client-api-c/classad_syntax_ex.h"
//#include "glite/ce/cream-client-api-c/string_manipulation.h"

namespace util = glite::ce::cream_client_api::util;

namespace glite {
  namespace ce {
    namespace cream_client_api {
      namespace util {

	class ConfigurationManager {

	  std::map<std::string, std::string> m_hash;
	  bool m_loaded;
	  
	public:
	  enum conf_format {
	    xml = 0, classad = 1, pairs = 2
	  };

	  ConfigurationManager(const ConfigurationManager&);
	  ConfigurationManager(const ConfigurationManager::conf_format& f=ConfigurationManager::pairs) : m_loaded(false), m_fmt(f) {}
	  ConfigurationManager& operator=(const ConfigurationManager&);
	  virtual ~ConfigurationManager() {}
	  
	  virtual std::string        getProperty(const std::string& s);// { return m_hash[s]; }
	  virtual void               setProperty(const std::string& s, const std::string& S) { m_hash[s]=S; }
	  virtual std::string        getProperty(const std::string& s, const std::string& def);/* {
	    if( m_hash[s].empty() ) return def;
	    else return m_hash[s];
	  }*/
	  virtual void          listProperties(FILE*);
	  virtual void          getKeys(std::vector<std::string>&);
	  virtual void          load(const char* filename) throw(util::file_ex&, util::classad_syntax_ex&);
	  /**
	   * When we load multiple files the attributes from different files
	   * are merged together in the internal hash table.
	   * Furthermore in case of attribute duplication, thanks to the map<> property,
	   * the attribute from i-th file overwrites the attribute with same name 
	   * specified in the (i-1)-th file.
	   */
	  virtual void          load(const std::vector<std::string>&) throw(util::file_ex&);
	  virtual void          store(const char* filename) throw (util::file_ex&) {}
	  
	private:
	  conf_format m_fmt;
	};
      }
    }
  }
}

#endif
