/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CreamProxy_Impl: All the implementation of the abstract class 
	        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CreamProxy_Impl.h,v 1.41.4.3 2008/08/25 14:06:38 adorigo Exp $
*/

#ifndef _GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_CREAMPROXYIMPL_
#define _GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_CREAMPROXYIMPL_

#include "glite/ce/cream-client-api-c/AbsCreamProxy.h"
#include "glite/ce/cream-client-api-c/JobIdWrapper.h"
#include "glite/ce/cream-client-api-c/auth_ex.h"

#include <map>
#include <utility>
#include <list>
#include <vector>

namespace glite {
  namespace ce {
    namespace cream_client_api {
      namespace soap_proxy {
	
	class JobDescriptionWrapper;
	class ResultWrapper;
	class JobFilterWrapper;
 	class ServiceInfoWrapper;
	
	class processorStatusResult {

	  AbsCreamProxy::StatusArrayResult* m_target;
	  
	public:
	  /**
	   *
	   * StatusArrayResult = map <string, pair< bool, StatusResult> >
	   * StatusResult      = union...
	   *
	   */
	  processorStatusResult( AbsCreamProxy::StatusArrayResult* target ) : m_target(target) {}
	  
	  void operator()( const CREAMTYPES__JobStatusResult* jr ) ;
	    
	};
	
	class processorRegisterResult {
	  
	  AbsCreamProxy::RegisterArrayResult* m_target;
	  
	public:
	  
	  /**
	   *
	   * RegisterArrayResult = map <string, pair< bool, RegisterResult> >
	   * RegisterResult      = union...
	   *
	   */
	  processorRegisterResult( AbsCreamProxy::RegisterArrayResult* target ) : m_target(target) {}
	    
	    void operator()( const CREAMTYPES__JobRegisterResult* jr ) ;
	    
	};
	
	class processorInfoResult {
	  
	  AbsCreamProxy::InfoArrayResult* m_target;
	  
	public:
	  
	  /**
	   *
	   * InfoArrayResult = map <string, pair< bool, InfoResult> >
	   * InfoResult      = union...
	   *
	   */
	  processorInfoResult( AbsCreamProxy::InfoArrayResult* target ) : m_target(target) {}
	    
	    //____________________________________________________________________________
	    void operator()( const CREAMTYPES__JobInfoResult* jr ) ; //  operator()
	    
	};
	
	class processorListResult {
	  
	  std::vector<JobIdWrapper> *m_target;
	  
	public:
	  processorListResult( std::vector<JobIdWrapper>* target ) : m_target( target ) {}
	    
	    void operator()( const CREAMTYPES__JobId* aJob );
	};

	//______________________________________________________________________________
	class CreamProxy_Register : public AbsCreamProxy 
	{
	  
	  const AbsCreamProxy::RegisterArrayRequest* m_jd_list;
	  AbsCreamProxy::RegisterArrayResult* m_target;
	  
	  CreamProxy_Register( const CreamProxy_Register& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Register( const RegisterArrayRequest*, RegisterArrayResult*, const int timeout = 30 ) throw();
	  
	  
	  
	public:
	  
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //								    AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  virtual ~CreamProxy_Register( void ) throw() {}
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Info : public AbsCreamProxy 
	{
	  
	  const JobFilterWrapper* m_jobFilter;
	  AbsCreamProxy::InfoArrayResult* m_target;
	  
	  CreamProxy_Info( const CreamProxy_Info& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Info( const JobFilterWrapper*, InfoArrayResult*, const int timeout = 30 ) throw();
	  
	public:
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  
	  ~CreamProxy_Info( void ) throw() {}
	};
	
	//______________________________________________________________________________
	// class CreamProxy_GetCEMonURL : public AbsCreamProxy 
	// {
	
	//   std::string* m_cemonURL;//_target;
	
	//   CreamProxy_GetCEMonURL( const CreamProxy_GetCEMonURL& ) throw() : AbsCreamProxy(30) {}
	//  protected:
	//   friend class CreamProxyFactory;
	//   CreamProxy_GetCEMonURL( std::string*, const int timeout = 30 ) throw();
	
	//  public:
	//   virtual void execute(const std::string& serviceURL) throw(BaseException&,
	//                                                             InvalidArgumentException&,
	//                                                             GridProxyDelegationException&,
	//                                                             JobSubmissionDisabledException&,
	//                                                             GenericException&,
	//                                                             AuthenticationException&,
	//                                                             AuthorizationException&,
	//                                                             DelegationException&,
	//                                                             InternalException&, 
	//                                                             ConnectionTimeoutException&,
	//                                                             auth_ex&);
	
	
	//   ~CreamProxy_GetCEMonURL( void ) throw() {}
	
	//   //virtual void garbage_collect( void ) const throw();
	// };
	
	
	//______________________________________________________________________________
	class CreamProxy_Start : public AbsCreamProxy 
	{
	  
	  CreamProxy_Start( const CreamProxy_Start& ) throw() : AbsCreamProxy(30) {}
	  
	  const JobFilterWrapper *m_jobFilter;
	  ResultWrapper    *m_result;
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Start( const JobFilterWrapper*, ResultWrapper*, const int timeout = 30 ) throw();
	  
	public:
	  
	  virtual ~CreamProxy_Start( void ) throw() {}
	  
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
							  	    soap_runtime_ex&);
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_List : public AbsCreamProxy 
	{
	  
	  CreamProxy_List( const CreamProxy_List& ) throw() : AbsCreamProxy(30) {}
	  
	  std::vector<JobIdWrapper>* m_result;
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_List( std::vector<JobIdWrapper>* result, const int timeout = 30 ) throw();
	  
	public:
	  
	  virtual ~CreamProxy_List( void ) throw() {}
	  
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Status : public AbsCreamProxy 
	{
	  
	  const JobFilterWrapper* m_jobFilter;
	  AbsCreamProxy::StatusArrayResult* m_target;
	  
	  CreamProxy_Status( const CreamProxy_Status& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Status( const JobFilterWrapper*, StatusArrayResult*, const int timeout = 30 ) throw();
	  
	public:
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  ~CreamProxy_Status( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Lease : public AbsCreamProxy 
	{
	  
	  const std::string m_leaseID;
	  const time_t      m_leaseTime;
	  std::pair<std::string, time_t>* m_resp_lease;
	  
	  CreamProxy_Lease( const CreamProxy_Lease& ) throw() : AbsCreamProxy(30), m_leaseID(""), m_leaseTime(0) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Lease( const std::pair<std::string, time_t>& Lease, std::pair<std::string, time_t>*, const int timeout = 30 ) throw();
	  
	public:
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  
	  virtual ~CreamProxy_Lease( void ) throw() { }
	  
	};
		
	//______________________________________________________________________________
	class CreamProxy_LeaseInfo : public AbsCreamProxy 
	{
	  
	  const std::string m_leaseID;
	  //const time_t      m_leaseTime;
	  std::pair<std::string, time_t>* m_resp_lease;
	  
	  CreamProxy_LeaseInfo( const CreamProxy_LeaseInfo& ) throw() 
	    : AbsCreamProxy(30), m_leaseID("") {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_LeaseInfo( const std::string& LeaseID, std::pair<std::string, time_t>*, const int timeout = 30 ) throw();
	  
	public:
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  
	  virtual ~CreamProxy_LeaseInfo( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Suspend : public AbsCreamProxy 
	{
	  
	  const JobFilterWrapper *m_jobFilter;
	  ResultWrapper          *m_result;
	  
	  CreamProxy_Suspend( const CreamProxy_Suspend& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Suspend( const JobFilterWrapper*, ResultWrapper*, const int timeout = 30 ) throw();
	  
	public:
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  
	  ~CreamProxy_Suspend( void ) throw() { }
	  
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Resume : public AbsCreamProxy 
	{
	  
	  const JobFilterWrapper *m_jobFilter;
	  ResultWrapper    *m_result;
	  
	  CreamProxy_Resume( const CreamProxy_Resume& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Resume( const JobFilterWrapper*, ResultWrapper*, const int timeout = 30 ) throw(); 
	public:
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  
	  ~CreamProxy_Resume( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Cancel : public AbsCreamProxy 
	{
	  
	  const JobFilterWrapper *m_jobFilter;
	  ResultWrapper    *m_result;
	  
	  CreamProxy_Cancel( const CreamProxy_Cancel& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Cancel( const JobFilterWrapper*, ResultWrapper*, const int timeout = 30 ) throw();
	  
	public:
	  
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  
	  ~CreamProxy_Cancel( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Purge : public AbsCreamProxy 
	{
	  
	  const JobFilterWrapper* m_jobFilter;
	  ResultWrapper*    m_result;
	  
	  CreamProxy_Purge( const CreamProxy_Purge& ) throw() : AbsCreamProxy(30) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Purge( const JobFilterWrapper*, ResultWrapper*, const int timeout = 30) throw();
	  
	public:
	  
	  virtual void execute(const std::string& serviceURL) throw(BaseException&,
								    InvalidArgumentException&,
								    GridProxyDelegationException&,
								    JobSubmissionDisabledException&,
								    GenericException&,
								    //AuthenticationException&,
								    AuthorizationException&,
								    DelegationException&,
								    InternalException&,
								    ConnectionTimeoutException&,
								    auth_ex&,
								    soap_ex&,
								    soap_runtime_ex&);
	  
	  ~CreamProxy_Purge( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_Delegate : public AbsCreamProxy 
	{
	  const std::string m_delegation_ID;
//	  std::string* m_certtxt;
	  
	  CreamProxy_Delegate( const CreamProxy_Delegate& ) throw() : AbsCreamProxy(30), m_delegation_ID("") {};
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_Delegate( const std::string& delegID, /*std::string*,*/ const int timeout = 30 ) throw();
	  
	public:
	  virtual void execute( const std::string& serviceURL) throw(BaseException&,
								     InvalidArgumentException&,
								     GridProxyDelegationException&,
								     JobSubmissionDisabledException&,
								     GenericException&,
								     //AuthenticationException&,
								     AuthorizationException&,
								     DelegationException&,
								     InternalException&,
								     ConnectionTimeoutException&,
								     auth_ex&,
								     soap_ex&,
								     soap_runtime_ex&);
	  
	  
	  virtual ~CreamProxy_Delegate( void ) throw() { };
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_ProxyRenew : public AbsCreamProxy 
	{
	  const std::string m_delegation_ID;
//	  std::string* m_certtxt;
	  
	  CreamProxy_ProxyRenew( const CreamProxy_ProxyRenew& ) throw() : AbsCreamProxy( 30 ) {}
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_ProxyRenew( const std::string& delegID/*, std::string* S_certtxt*/, const int timeout ) throw();
	  
	public:
	  virtual void execute( const std::string& serviceURL) throw(BaseException&,
								     InvalidArgumentException&,
								     GridProxyDelegationException&,
								     JobSubmissionDisabledException&,
								     GenericException&,
								     //AuthenticationException&,
								     AuthorizationException&,
								     DelegationException&,
								     InternalException&,
								     ConnectionTimeoutException&,
								     auth_ex&,
								     soap_ex&,
								     soap_runtime_ex&);
	  
	  
	  ~CreamProxy_ProxyRenew( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	class CreamProxy_ServiceInfo : public AbsCreamProxy 
	{
	  
	  //std::string* m_info;
	  ServiceInfoWrapper* m_target;
	  

	  CreamProxy_ServiceInfo( const CreamProxy_ServiceInfo& ) throw() 
	    : AbsCreamProxy(30) { }
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_ServiceInfo( ServiceInfoWrapper*, const int) throw();
	  
	public:
	  
	  virtual void execute( const std::string& ) throw(BaseException&,
							   InvalidArgumentException&,
							   GridProxyDelegationException&,
							   JobSubmissionDisabledException&,
							   GenericException&,
							   //AuthenticationException&,
							   AuthorizationException&,
							   DelegationException&,
							   InternalException&,
							   ConnectionTimeoutException&,
							   auth_ex&,
							   soap_ex&,
							   soap_runtime_ex&);
	  
	  
	  ~CreamProxy_ServiceInfo( void ) throw() { }
	  
	};
		
	//______________________________________________________________________________
	class CreamProxy_AcceptNewJobSubmissions : public AbsCreamProxy 
	{
	  
	  bool m_set;

	  CreamProxy_AcceptNewJobSubmissions( const CreamProxy_AcceptNewJobSubmissions& ) throw() 
	    : AbsCreamProxy(30) { }
	  
	protected:
	  friend class CreamProxyFactory;
	  CreamProxy_AcceptNewJobSubmissions( const bool, const int ) throw();
	  
	public:
	  
	  virtual void execute( const std::string& ) throw(BaseException&,
							   InvalidArgumentException&,
							   GridProxyDelegationException&,
							   JobSubmissionDisabledException&,
							   GenericException&,
							   //AuthenticationException&,
							   AuthorizationException&,
							   DelegationException&,
							   InternalException&,
							   ConnectionTimeoutException&,
							   auth_ex&,
							   soap_ex&,
							   soap_runtime_ex&);
	  
	  
	  ~CreamProxy_AcceptNewJobSubmissions( void ) throw() { }
	  
	};
	
	//______________________________________________________________________________
	// class CreamProxy_EnableJS : public AbsCreamProxy 
	// {
	//   CreamProxy_EnableJS( const CreamProxy_EnableJS& ) throw() : AbsCreamProxy(30) {}
	
	//  protected:
	//   friend class CreamProxyFactory;
	//   CreamProxy_EnableJS( const int timeout = 30) throw() : AbsCreamProxy( timeout ) {}
	
	//  public:
	
	//   virtual void execute( const std::string& serviceURL) throw(BaseException&,
	// 							     InvalidArgumentException&,
	// 							     GridProxyDelegationException&,
	// 							     JobSubmissionDisabledException&,
	// 							     GenericException&,
	// 							     AuthenticationException&,
	// 							     AuthorizationException&,
	// 							     DelegationException&,
	// 							     InternalException&,
	// 							     ConnectionTimeoutException&,
	// 							     auth_ex&);
	
	//   ~CreamProxy_EnableJS( void ) throw() {}
	
	// };
	
	//______________________________________________________________________________
	// class CreamProxy_DisableJS : public AbsCreamProxy 
	// {
	//   CreamProxy_DisableJS( const CreamProxy_DisableJS& ) throw() : AbsCreamProxy(30) {}
	
	//  protected:
	//   friend class CreamProxyFactory;
	//   CreamProxy_DisableJS( const int timeout = 30) throw() : AbsCreamProxy( timeout ) {}
	
	//  public:
	
	//   virtual void execute( const std::string& serviceURL) throw(BaseException&,
	// 							     InvalidArgumentException&,
	// 							     GridProxyDelegationException&,
	// 							     JobSubmissionDisabledException&,
	// 							     GenericException&,
	// 							     AuthenticationException&,
	// 							     AuthorizationException&,
	// 							     DelegationException&,
	// 							     InternalException&,
	// 							     ConnectionTimeoutException&,
	// 							     auth_ex&);
	
	//   ~CreamProxy_DisableJS( void ) throw() {}
	
	
	//  };
	
	//______________________________________________________________________________
	// class CreamProxy_CheckJS : public AbsCreamProxy 
	// {
	
	//   CreamProxy_CheckJS( const CreamProxy_CheckJS& ) throw() : AbsCreamProxy(30) {}
	
	//   bool *m_accept;
	
	//  protected:
	//   friend class CreamProxyFactory;
	//   CreamProxy_CheckJS( bool* response, const int timeout = 30) throw() : AbsCreamProxy( timeout ), m_accept( response ) {}
	
	//  public:
	
	//   virtual void execute( const std::string& serviceURL) throw(BaseException&,
	// 							     InvalidArgumentException&,
	// 							     GridProxyDelegationException&,
	// 							     JobSubmissionDisabledException&,
	// 							     GenericException&,
	// 							     AuthenticationException&,
	// 							     AuthorizationException&,
	// 							     DelegationException&,
	// 							     InternalException&,
	// 							     ConnectionTimeoutException&,
	// 							     auth_ex&);
	
	
	//   ~CreamProxy_CheckJS( void ) throw() {}
	
	//   //virtual void garbage_collect( void ) const throw();
	// };
	
	
      }
    }
  }
}

#endif
