 /*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        JobIdWrapper: a wrapper for the new class JobDescription
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: JobIdWrapper.h,v 1.14.4.3 2008/08/29 08:25:58 adorigo Exp $
*/

#ifndef GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_JOBIDWRAPPER
#define GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_JOBIDWRAPPER

#include "glite/ce/cream-client-api-c/cream_client_soapH.h"
#include "glite/ce/cream-client-api-c/cream_client_soapStub.h"

#include "glite/ce/cream-client-api-c/JobPropertyWrapper.h"

#include <string>
#include <vector>
#include <map>
#include <list>

namespace glite {
namespace ce {
namespace cream_client_api {
namespace soap_proxy {
  
    //________________________________________________________________________

  // /// \class A Class to Wrap the Cream Job ID
  /**
     \class JobIdWrapper
     \brief A Class to Wrap the Cream Job

     This class is a structure containing the Cream Job ID string, the CREAM URL the job has been registered to, and a vector
     of properties, where each property is a couple of strings (name, value), and finally the result of the operation that has been invoked (JobRegister, JobInfo, JobStart, etc...).\n In fact this structure is tipically used as member of complex output of several CREAM remote operations.
  */
    class JobIdWrapper : public CREAMTYPES__JobId {    
            
    friend class JobInfoWrapper;
    friend class JobFilterWrapper;
    friend class JobStatusWrapper;
    friend class ResultWrapper;
    friend class processorRegisterResult;
    friend class processorListResult;

    protected:	
        void clone( const CREAMTYPES__JobId* other ) throw();
        void destroy( void ) throw(); 
	JobIdWrapper( const CREAMTYPES__JobId* other );
       
    public:
        /**
	   Creates a JobIdWrapper object. 
	   
	   \param CreamJobId const std::string& An unique identifier returned by the CREAM CE web service
	   \param CreamURL const std::string& The URL of the CREAM CE web service where this job comes from
	   \param prop const std::vector<JobPropertyWrapper>& A vector of JobPropertyWrapper

	   \see JobPropertyWrapper
         */
        JobIdWrapper( const std::string& CreamJobId, const std::string& CreamURL, const std::vector<JobPropertyWrapper>& prop) throw();
        
        
	/**
	   Simple constructor; this is needed when a JobIdWrapper object must be put in a vector/list or other containers
	*/
        JobIdWrapper( ) throw() 
        {

        }

        /**
	   Copy assignment operator
	*/
        JobIdWrapper& operator=( const JobIdWrapper& aJobId ) throw();
        
        /**
         * Copy Constructor
         */
        JobIdWrapper( const JobIdWrapper& aJob ) throw() ;
        
        /**
         * Class destructor
         */
        virtual ~JobIdWrapper() throw() ;
        
	/**
	   Gets the local Cream Job unique identifier
	   
	   \return std::string
	*/
        std::string getCreamJobID( void ) const { return id; }

	/**
	   Gets the local Cream URL the job has been submitted to

	   \return std::string
	*/
        std::string getCreamURL( void ) const { return creamURL; }

	
        void getProperties( std::map< std::string, std::string >& target );

	/**
	   The types of result of a job operation (JogRegister, JobCancel, etc...)
	*/
	enum RESULT {
	  OK, /**< The operation related to the current Job is succesfull */
	  JOBUNKNOWN, /**< The current Job is unknown to the CREAM CE remote service */
	  JOBSTATUSINVALID, /**< The current Job's status is incompatible with the requested operation */
	  DELEGATIONIDMISMATCH, /**< The delegation ID used for job registration is not recognized by remote CREAM CE service */
	  DATEMISMATCH, /**< The date specified for the operation is not valid */
	  LEASEIDMISMATCH,/**< The Lease ID used for job registration is not recognized by remote CREAM CE service */
	  DELEGATIONPROXYERROR,/**< An error occurred during proxy delegation */
	  GENERIC/**< A generic error */
	};
    };
	  
}
}
}
}

#endif
