 /*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        JobIdWrapper: a wrapper for the new class JobDescription
        
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: JobPropertyWrapper.h,v 1.3.4.2 2008/08/11 13:20:52 adorigo Exp $
*/

#ifndef GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_JPW
#define GLITE_CE_CREAM_CLIENT_API_SOAPPROXY_JPW

//#include "CREAM_CLIENTH.h"
//#include "CREAM_CLIENTStub.h"

#include "glite/ce/cream-client-api-c/cream_client_soapH.h"
#include "glite/ce/cream-client-api-c/cream_client_soapStub.h"

#include <string>

namespace glite {
namespace ce {
namespace cream_client_api {
namespace soap_proxy {

  /// A wrapper class for the property
  /**
     Some information job-related and service-related are exchanged between  
     CREAM and its clients as couple of strings (name, value) that are structured in the property class.

     \see JobIdWrapper
  */
  class JobPropertyWrapper : public CREAMTYPES__Property {
    
  protected:

    JobPropertyWrapper& operator=( const JobPropertyWrapper& aProp ) throw ()
      { 
	if ( &aProp != this ) {
	  name  = aProp.name;
	  value = aProp.value;
	}
	return *this; 
      }

    JobPropertyWrapper( const CREAMTYPES__Property* other ) 
      {
	if ( other ) {
	  name = other->name;
	  value = other->value;
	}
      }
    
  public:
    /**
       Class constructor
       
       \param n The name of a property
       \param v The value of the property
    */
    JobPropertyWrapper( const std::string& n, const std::string& v) throw() 
      { name = n; value = v; }
    
    
    

    /**
       Clas destructor
    */
    virtual ~JobPropertyWrapper() throw() {}
    
    /**
       Return the name of the property

       \return std::string
    */
    std::string getPropertyName( void ) const { return name; }

    /**
       Return the value of the property
       
       \return std::string
    */
    std::string getPropertyValue( void ) const { return value; }
    
  };
  
}
}
}
}

#endif
