/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        certUtil: utilities to extract useful info from a proxy certificate
	
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: certUtil.h,v 1.9.4.1.4.2 2008/12/16 15:26:46 adorigo Exp $
*/

#ifndef _GLITE_CE_CREAM_CLIENT_API_UTIL_CERTUTIL_H_
#define _GLITE_CE_CREAM_CLIENT_API_UTIL_CERTUTIL_H_
 
#include "glite/ce/cream-client-api-c/auth_ex.h"
#include <string>
#include <glite/security/voms/voms_api.h>
#include <openssl/pem.h>

namespace glite {
namespace ce {
namespace cream_client_api {

   class helper_BIOnew { 
     BIO *in; 
   public: 
     helper_BIOnew(BIO* infile) : in(infile) {} 
     ~helper_BIOnew() {  
       if(in) { 
 	BIO_set_close(in, BIO_CLOSE);     
 	BIO_free(in); 
       }  
     } 
   }; 
  
   class helper_X509new { 
     X509* x; 
   public: 
     helper_X509new(X509* xin) : x(xin) {} 
     ~helper_X509new() { if(x) X509_free(x); } 
   }; 
  
   class helper_skX509new { 
     STACK_OF(X509) *sk; 
   public: 
     helper_skX509new(STACK_OF(X509)* x) : sk(x) {} 
     ~helper_skX509new() { 
       if( sk ) { 
 	sk_X509_pop_free( sk, X509_free ); 
       } 
     } 
   }; 

  namespace certUtil {
     X509* read_BIO( const std::string& pxfile )  
       throw( glite::ce::cream_client_api::soap_proxy::auth_ex& ); 

      std::string getDN(const std::string& proxyfile) 
       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);

/*     std::string getDNFQAN(const std::string& proxyfile) */
/*       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&); */
    

/*     std::string getCertIssuer(const std::string& proxyfile) */
/*       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&); */
        

     std::string getCertSubj(const std::string& proxyfile) 
       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&); 
            

/*     std::string getVOName( const std::string& proxyfile) */
/*       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&); */
    

     const long int getProxyTimeLeft( const std::string& proxyfile )
       throw (glite::ce::cream_client_api::soap_proxy::auth_ex&);
    
 
/*     X509* getX509Fromfile(const std::string& proxyfile) */
/*       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&); */
    

/*     STACK_OF(X509) *load_chain(char *certfile) throw(); */
    
/*     void getFQANs( const std::string&, std::vector<std::string>& ) */
/*       throw(glite::ce::cream_client_api::soap_proxy::auth_ex&); */

    void generateUniqueID(std::string& target) throw();
    
    void bintostring( unsigned char* buf, size_t len, std::string& );
    
    int GRSTx509MakeProxyCert_local(char **proxychain, FILE *debugfp, 
                                    char *reqtxt, char *cert, char *key, int minutes);

    time_t ASN1_UTCTIME_get(const ASN1_UTCTIME*);
  };
  
 }
 }
}

#endif
